function [mse fin] = bp(data, target, parm)

# err = bp_bak(data, target, parm)
#
# on-line backpropagtion
#
# data =(observations * input_dimension)
# w_ji = weight from input i to hidden unit j;  (hidden, i_dimension+1)
# w_kj = weight from hidden j to output unit k; (output, hidden+1)
#
# last column of both w_ji and w_kj hold the unit bias
#
# use the tanh() activation function on the hidden layer and the
# sigmoid() in the output layer
#
  
global w_ji w_kj ow_ji ow_kj odw_ji odw_kj mse_hist

[nr nc] = size(data);				# number os patterns
[nh ni] = size(w_ji); ni=ni-1;		# number of hidden and input units
no = rows(w_kj);					# number of output units

w_d_ji = zeros(nh,ni+1);
w_d_kj = zeros(no,nh+1);
w_dz_ji = zeros(nh,1);
w_dz_kj = zeros(no,1);
	
eta = parm.eta;				# eta
epoch = parm.epoch;			# stop after number of epochs
min_mse = parm.min_mse;		# or if min_mse is obtained
h_act = parm.h_act;			# hidden units activation function
h_d_act = parm.h_d_act;		# hidden units derivative of activation function
o_act = parm.o_act;			# output units activation function
o_d_act = parm.o_d_act;		# output units derivative of activation function
fin = 0;

target = target';
data = [data ones(nr,1)]';				# last column hold weights to bias units

for j=1:epoch
	sum_mse = 0;
	sw_ji = zeros(nh,ni+1);		# acumulated weights changes on input-hidden layer 
	sw_kj = zeros(no,nh+1);		# acumulated weights changes on hidden-output layer 
for i=1:nr
	x_i = data(:,i);
	a_j = feval(h_act,  w_ji * x_i);	# activation of hidden units -- tanh(.)
	a_k = feval(o_act, w_kj * [a_j; 1]);# activation of out units -- sigmoid(.)
	e_k = a_k .- target(:,i);			# difference between target and net output
	sum_mse = sum_mse + (e_k' * e_k);	# mse error
	er_k = feval(o_d_act, a_k) .* e_k;	# error at output nodes to back propagate
	er_j = [feval(h_d_act, a_j) ;1] .* ( w_kj' * er_k);	# error at hidden nodes to bp
	sw_kj = sw_kj .+ (eta .* (er_k * [a_j;1]'));		# acumulated weights changes
	sw_ji = sw_ji .+ (eta .* (er_j(1:nh) * x_i'));						
endfor

	w_kj = w_kj .- sw_kj;						# final weights
	w_ji = w_ji .- sw_ji;
	
	mse_hist = [mse_hist sum_mse/nr];

	if (sum_mse/nr <= min_mse)
		fin = 1;
		break
	endif
	
endfor

mse = mse_hist;
	
endfunction
