function [mse fin] = bp_ol(data, target, parm)

# err = bp_bak(data, target, parm)
#
# on-line backpropagtion
#
# data =(observations * input_dimension)
# w_ji = weight from input i to hidden unit j;  (hidden, i_dimension+1)
# w_kj = weight from hidden j to output unit k; (output, hidden+1)
#
# last column of both w_ji and w_kj hold the unit bias
  
global w_ji w_kj ow_ji ow_kj odw_ji odw_kj mse_hist

[nr nc] = size(data);
[nh ni] = size(w_ji); ni=ni-1;
no = rows(w_kj);

w_d_ji = zeros(nh,ni+1);
w_d_kj = zeros(no,nh+1);
w_dz_ji = zeros(nh,1);
w_dz_kj = zeros(no,1);

eta = parm.eta;             # backprop eta parameter    (0.001:1--0.1)
mu = parm.mu;               # momentum parameter        (0:1--0.5)
lambda = parm.lambda;       # weight decay parameter    (0:0.01--0.00001)
								# if positive, use Hinton wd formula,
								# if negative, use Weigend wd.
flat_s = parm.flat_s;       # flat-spot elimination parameter (Fahlman) (0:0.1-$

epoch = parm.epoch;         # stop after number of epochs
min_mse = parm.min_mse;     # or if min_mse is obtained

h_act = parm.h_act;         # hidden units activation function
h_d_act = parm.h_d_act;     # hidden units derivative of activation function
o_act = parm.o_act;         # output units activation function
o_d_act = parm.o_d_act;     # output units derivative of activation function

fin = 0;

target = target';
data = [data ones(nr,1)]';

for j=1:epoch
	sum_mse = sum_mce = 0;
	[dummy,ix] = sort(rand(1,nr));  data = data(:,ix);	# shuffle data
for i=1:nr
	x_i = data(:,i);
	
	a_j = feval(h_act,  w_ji * x_i);    # activation of hidden units
    a_k = feval(o_act, w_kj * [a_j; 1]);# activation of out units    
	e_k = a_k .- target(:,i);			# difference between target and net output
	sum_mse = sum_mse + (e_k' * e_k);					# mse error
	er_k = (feval(o_d_act, a_k) .+ flat_s) .* e_k;
	er_j = ([feval(h_d_act, a_j);1] .+ flat_s) .* ( w_kj' * er_k);
	dw_kj = (mu .* odw_kj) .- (eta .* (er_k * [a_j;1]'));	# delta_W of hidden-output layers
	dw_ji = (mu .* odw_ji) .- (eta .* (er_j(1:nh) * x_i'));	# delta_W of input-hidden layers
	if (lambda)
		w_d_ji = lambda .* w_ji;						# Hinton weight decay
		w_d_kj = lambda .* w_kj;	
	if (lambda < 0)										# just an way to signal Weigend wd
		w_d_ji = - w_d_ji ./ ((1 .+ w_ji .^ 2) .^2);	# Weigend weight decay
		w_d_kj = - w_d_kj ./ ((1 .+ w_kj .^ 2) .^2);	
	endif
		w_d_ji(:,ni+1) = w_dz_ji;						# don't decay biases
		w_d_kj(:,nh+1) = w_dz_kj;
	endif
	w_kj = w_kj .+ dw_kj .- w_d_kj;						# final weights
	w_ji = w_ji .+ dw_ji .- w_d_ji;						
	odw_kj = dw_kj;										# old delta W
	odw_ji = dw_ji;
endfor

	mse_hist = [mse_hist sum_mse/nr];
	if (sum_mse/nr <= min_mse)
		fin = 1;
		break
	endif
	
#	mse_s = columns(mse_hist);
#	if (mse_s > 2)
#		t = mse_hist(mse_s-1);
#		c = mse_hist(mse_s);
#		if ( c > t*1.1 )	# error increase, diminish eta
#			eta = eta*0.5;
#			w_ji = ow_ji;
#			w_kj = ow_kj;
#			odw_ji = zeros(nh,ni+1);
#			odw_kj = zeros(no,nh+1);
#			mse_hist(mse_s)=[];
#		else #if (c < t*0.9) 	# error decrease, increase eta
#			eta = eta*1.1;
#			ow_ji = w_ji;			
#			ow_kj = w_kj;
#            odw_ji = dw_ji;
#           odw_kj = dw_kj;
#		endif
#	endif
	
endfor

mse = mse_hist;
#mce = mce_hist;
	
endfunction
